package sun.awt.windows;

import java.util.Collections;
import java.util.HashMap;
import java.awt.image.DataBufferInt;
import java.io.ByteArrayOutputStream;
import java.util.ArrayList;
import java.awt.Graphics2D;
import java.awt.image.WritableRaster;
import sun.awt.image.ImageRepresentation;
import java.awt.image.DataBufferByte;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.awt.image.Raster;
import java.awt.image.ComponentColorModel;
import java.awt.color.ColorSpace;
import sun.awt.image.ToolkitImage;
import java.awt.Image;
import java.util.Arrays;
import java.io.File;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.net.URL;
import java.io.InputStream;
import java.io.IOException;
import java.awt.datatransfer.Transferable;
import java.util.SortedMap;
import java.awt.datatransfer.FlavorTable;
import java.awt.datatransfer.DataFlavor;
import sun.awt.datatransfer.ToolkitThreadBlockedHandler;
import java.awt.image.DirectColorModel;
import java.util.Map;
import sun.awt.datatransfer.DataTransferer;

final class WDataTransferer extends DataTransferer
{
    private static final String[] predefinedClipboardNames;
    private static final Map<String, Long> predefinedClipboardNameMap;
    public static final int CF_TEXT = 1;
    public static final int CF_METAFILEPICT = 3;
    public static final int CF_DIB = 8;
    public static final int CF_ENHMETAFILE = 14;
    public static final int CF_HDROP = 15;
    public static final int CF_LOCALE = 16;
    public static final long CF_HTML;
    public static final long CFSTR_INETURL;
    public static final long CF_PNG;
    public static final long CF_JFIF;
    public static final long CF_FILEGROUPDESCRIPTORW;
    public static final long CF_FILEGROUPDESCRIPTORA;
    private static final Long L_CF_LOCALE;
    private static final DirectColorModel directColorModel;
    private static final int[] bandmasks;
    private static WDataTransferer transferer;
    private final ToolkitThreadBlockedHandler handler;
    private static final byte[] UNICODE_NULL_TERMINATOR;
    
    private WDataTransferer() {
        this.handler = new WToolkitThreadBlockedHandler();
    }
    
    static synchronized WDataTransferer getInstanceImpl() {
        if (WDataTransferer.transferer == null) {
            WDataTransferer.transferer = new WDataTransferer();
        }
        return WDataTransferer.transferer;
    }
    
    @Override
    public SortedMap<Long, DataFlavor> getFormatsForFlavors(DataFlavor[] flavors, FlavorTable map) {
        SortedMap<Long, DataFlavor> formatsForFlavors = super.getFormatsForFlavors(flavors, map);
        formatsForFlavors.remove(WDataTransferer.L_CF_LOCALE);
        return formatsForFlavors;
    }
    
    @Override
    public String getDefaultUnicodeEncoding() {
        return "utf-16le";
    }
    
    @Override
    public byte[] translateTransferable(Transferable transferable, DataFlavor dataFlavor, long n) throws IOException {
        byte[] array;
        if (n == WDataTransferer.CF_HTML) {
            if (transferable.isDataFlavorSupported(DataFlavor.selectionHtmlFlavor)) {
                array = super.translateTransferable(transferable, DataFlavor.selectionHtmlFlavor, n);
            }
            else if (transferable.isDataFlavorSupported(DataFlavor.allHtmlFlavor)) {
                array = super.translateTransferable(transferable, DataFlavor.allHtmlFlavor, n);
            }
            else {
                array = HTMLCodec.convertToHTMLFormat(super.translateTransferable(transferable, dataFlavor, n));
            }
        }
        else {
            array = super.translateTransferable(transferable, dataFlavor, n);
        }
        return array;
    }
    
    @Override
    public Object translateStream(InputStream str, DataFlavor flavor, long format, Transferable localeTransferable) throws IOException {
        if (format == WDataTransferer.CF_HTML && flavor.isFlavorTextType()) {
            str = new HTMLCodec(str, EHTMLReadMode.getEHTMLReadMode(flavor));
        }
        return super.translateStream(str, flavor, format, localeTransferable);
    }
    
    @Override
    public Object translateBytes(byte[] bytes, DataFlavor dataFlavor, long format, Transferable localeTransferable) throws IOException {
        if (format == WDataTransferer.CF_FILEGROUPDESCRIPTORA || format == WDataTransferer.CF_FILEGROUPDESCRIPTORW) {
            if (bytes == null || !DataFlavor.javaFileListFlavor.equals(dataFlavor)) {
                throw new IOException("data translation failed");
            }
            String[] split = new String(bytes, 0, bytes.length, "UTF-16LE").split("\u0000");
            if (0 == split.length) {
                return null;
            }
            File[] a = new File[split.length];
            for (int i = 0; i < split.length; ++i) {
                (a[i] = new File(split[i])).deleteOnExit();
            }
            return Arrays.asList(a);
        }
        else if (format == WDataTransferer.CFSTR_INETURL && URL.class.equals(dataFlavor.getRepresentationClass())) {
            String defaultTextCharset = getDefaultTextCharset();
            if (localeTransferable != null && localeTransferable.isDataFlavorSupported(WDataTransferer.javaTextEncodingFlavor)) {
                try {
                    defaultTextCharset = new String((byte[])(byte[])localeTransferable.getTransferData(WDataTransferer.javaTextEncodingFlavor), "UTF-8");
                }
                catch (UnsupportedFlavorException ex) {}
            }
            return new URL(new String(bytes, defaultTextCharset));
        }
        else {
            return super.translateBytes(bytes, dataFlavor, format, localeTransferable);
        }
    }
    
    @Override
    public boolean isLocaleDependentTextFormat(long n) {
        return n == 1L || n == WDataTransferer.CFSTR_INETURL;
    }
    
    @Override
    public boolean isFileFormat(long n) {
        return n == 15L || n == WDataTransferer.CF_FILEGROUPDESCRIPTORA || n == WDataTransferer.CF_FILEGROUPDESCRIPTORW;
    }
    
    @Override
    protected Long getFormatForNativeAsLong(String s) {
        Long value = (Long)WDataTransferer.predefinedClipboardNameMap.get(s);
        if (value == null) {
            value = Long.valueOf(registerClipboardFormat(s));
            return value;
        }
        return value;
    }
    
    @Override
    protected String getNativeForFormat(long n) {
        return (n < (long)WDataTransferer.predefinedClipboardNames.length) ? WDataTransferer.predefinedClipboardNames[(int)n] : getClipboardFormatName(n);
    }
    
    @Override
    public ToolkitThreadBlockedHandler getToolkitThreadBlockedHandler() {
        return this.handler;
    }
    
    private static native long registerClipboardFormat(String p0);
    
    private static native String getClipboardFormatName(long p0);
    
    @Override
    public boolean isImageFormat(long n) {
        return n == 8L || n == 14L || n == 3L || n == WDataTransferer.CF_PNG || n == WDataTransferer.CF_JFIF;
    }
    
    @Override
    protected byte[] imageToPlatformBytes(Image image, long n) throws IOException {
        String mimeType = null;
        if (n == WDataTransferer.CF_PNG) {
            mimeType = "image/png";
        }
        else if (n == WDataTransferer.CF_JFIF) {
            mimeType = "image/jpeg";
        }
        if (mimeType != null) {
            return this.imageToStandardBytes(image, mimeType);
        }
        int w;
        int h;
        if ((image instanceof ToolkitImage)) {
            ImageRepresentation imageRep = ((ToolkitImage)image).getImageRep();
            imageRep.reconstruct(32);
            w = imageRep.getWidth();
            h = imageRep.getHeight();
        }
        else {
            w = image.getWidth(null);
            h = image.getHeight(null);
        }
        int n2 = w * 3 % 4;
        int n3 = (n2 > 0) ? (4 - n2) : 0;
        ColorSpace instance = ColorSpace.getInstance(1000);
        int[] bits = { 8, 8, 8 };
        int[] bandOffsets = { 2, 1, 0 };
        ComponentColorModel cm = new ComponentColorModel(instance, bits, false, false, 1, 0);
        WritableRaster interleavedRaster = Raster.createInterleavedRaster(0, w, h, w * 3 + n3, 3, bandOffsets, null);
        BufferedImage o = new BufferedImage(cm, interleavedRaster, false, null);
        AffineTransform o2 = new AffineTransform(1.0f, 0.0f, 0.0f, -1.0f, 0.0f, (float)h);
        Graphics2D graphics = o.createGraphics();
        try {
            graphics.drawImage(image, o2, null);
        }
        finally {
            graphics.dispose();
        }
        return this.imageDataToPlatformImageBytes(((DataBufferByte)interleavedRaster.getDataBuffer()).getData(), w, h, n);
    }
    
    @Override
    protected ByteArrayOutputStream convertFileListToBytes(ArrayList<String> list) throws IOException {
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        if (list.isEmpty()) {
            byteArrayOutputStream.write(WDataTransferer.UNICODE_NULL_TERMINATOR);
        }
        else {
            for (int i = 0; i < list.size(); ++i) {
                byte[] bytes = ((String)list.get(i)).getBytes(this.getDefaultUnicodeEncoding());
                byteArrayOutputStream.write(bytes, 0, bytes.length);
                byteArrayOutputStream.write(WDataTransferer.UNICODE_NULL_TERMINATOR);
            }
        }
        byteArrayOutputStream.write(WDataTransferer.UNICODE_NULL_TERMINATOR);
        return byteArrayOutputStream;
    }
    
    private native byte[] imageDataToPlatformImageBytes(byte[] p0, int p1, int p2, long p3);
    
    @Override
    protected Image platformImageBytesToImage(byte[] bytes, long n) throws IOException {
        String mimeType = null;
        if (n == WDataTransferer.CF_PNG) {
            mimeType = "image/png";
        }
        else if (n == WDataTransferer.CF_JFIF) {
            mimeType = "image/jpeg";
        }
        if (mimeType != null) {
            return this.standardImageBytesToImage(bytes, mimeType);
        }
        int[] platformImageBytesToImageData = this.platformImageBytesToImageData(bytes, n);
        if (platformImageBytesToImageData == null) {
            throw new IOException("data translation failed");
        }
        int size = platformImageBytesToImageData.length - 2;
        int n2 = platformImageBytesToImageData[size];
        return new BufferedImage(WDataTransferer.directColorModel, Raster.createPackedRaster(new DataBufferInt(platformImageBytesToImageData, size), n2, platformImageBytesToImageData[size + 1], n2, WDataTransferer.bandmasks, null), false, null);
    }
    
    private native int[] platformImageBytesToImageData(byte[] p0, long p1) throws IOException;
    
    @Override
    protected native String[] dragQueryFile(byte[] p0);
    
    static {
        predefinedClipboardNames = new String[] { "", "TEXT", "BITMAP", "METAFILEPICT", "SYLK", "DIF", "TIFF", "OEM TEXT", "DIB", "PALETTE", "PENDATA", "RIFF", "WAVE", "UNICODE TEXT", "ENHMETAFILE", "HDROP", "LOCALE", "DIBV5" };
        HashMap m = new HashMap(WDataTransferer.predefinedClipboardNames.length, 1.0f);
        for (int i = 1; i < WDataTransferer.predefinedClipboardNames.length; ++i) {
            m.put(WDataTransferer.predefinedClipboardNames[i], Long.valueOf((long)i));
        }
        predefinedClipboardNameMap = Collections.synchronizedMap(m);
        CF_HTML = registerClipboardFormat("HTML Format");
        CFSTR_INETURL = registerClipboardFormat("UniformResourceLocator");
        CF_PNG = registerClipboardFormat("PNG");
        CF_JFIF = registerClipboardFormat("JFIF");
        CF_FILEGROUPDESCRIPTORW = registerClipboardFormat("FileGroupDescriptorW");
        CF_FILEGROUPDESCRIPTORA = registerClipboardFormat("FileGroupDescriptor");
        L_CF_LOCALE = (Long)WDataTransferer.predefinedClipboardNameMap.get(WDataTransferer.predefinedClipboardNames[16]);
        directColorModel = new DirectColorModel(24, 16711680, 65280, 255);
        bandmasks = new int[] { WDataTransferer.directColorModel.getRedMask(), WDataTransferer.directColorModel.getGreenMask(), WDataTransferer.directColorModel.getBlueMask() };
        UNICODE_NULL_TERMINATOR = new byte[] { 0, 0 };
    }
}
